"""Common stuff used by multiple parts of OGS Container Maker"""

from __future__ import absolute_import

import sys

from ogscm.common import package_manager

g_package_manager = package_manager.SYSTEM


def set_package_manager(pm):
    this = sys.modules[__name__]

    if pm == "easybuild":
        this.g_package_manager = package_manager.EASYBUILD
    elif pm == "guix":
        this.g_package_manager = package_manager.GUIX
    elif pm == "system":
        this.g_package_manager = package_manager.SYSTEM
    elif pm == "off":
        this.g_package_manager = package_manager.OFF
    else:
        RuntimeError("Invalid package manager!")



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""OGS base building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import hpccm.config

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.building_blocks.pip import pip
from hpccm.building_blocks.python import python
from hpccm.common import linux_distro
from hpccm.primitives.comment import comment
from hpccm.primitives.shell import shell
from hpccm.primitives.environment import environment


class ogs_base(bb_base):
    """OGS base building block"""

    def __init__(self, **kwargs):
        """Initialize building block"""
        super(ogs_base, self).__init__()

        self.__ospackages = kwargs.get("ospackages", [])

        self.__commands = []  # Filled in by __setup()
        self.__wd = "/var/tmp"  # working directory

        self.__setup()

        self.__instructions()

    def __instructions(self):
        """String representation of the building block"""
        self += comment(__doc__, reformat=False)
        self += python(devel=True, python2=False)
        self += packages(ospackages=["python3-pip", "python3-virtualenv"])
        self += shell(
            commands=["pip3 install --break-system-packages pre-commit cmake-format"]
        )
        self += packages(ospackages=self.__ospackages, epel=True)
        self += shell(commands=self.__commands)

    def __setup(self):
        self.__ospackages.extend(["git"])

        dist = "deb"
        if hpccm.config.g_linux_distro == linux_distro.CENTOS:
            dist = "rpm"
            self.__commands.extend(
                [
                    "wget https://github.com/ninja-build/ninja/releases/download/v1.10.0/ninja-linux.zip",
                    "unzip ninja-linux.zip",
                    "mv ninja /usr/local/bin",
                    "rm ninja-linux.zip",
                ]
            )
        else:
            self.__ospackages.extend(["ninja-build"])

        if hpccm.config.g_ctype == hpccm.container_type.SINGULARITY:
            self.__ospackages.append("locales")
            self.__commands.extend(
                [
                    'echo "LC_ALL=en_US.UTF-8" >> /etc/environment',
                    'echo "en_US.UTF-8 UTF-8" >> /etc/locale.gen',
                    'echo "LANG=en_US.UTF-8" > /etc/locale.conf',
                    "locale-gen en_US.UTF-8",
                ]
            )

        # Common directories
        self.__commands.append("mkdir -p /apps /scratch /lustre /work /projects /data")

        # virtualenv
        if hpccm.config.g_linux_distro == linux_distro.UBUNTU:
            self.__ospackages.extend(["python3-venv", "python-is-python3"])
        else:
            self.__ospackages.append("python3-virtualenv")
            self.__commands.append("alternatives --set python /usr/bin/python3")

    def runtime(self, _from="0"):
        instructions = [
            comment(__doc__, reformat=False),
            python(devel=True, python2=False).runtime(),
            packages(ospackages=["python3-virtualenv"]),
            shell(commands=["mkdir -p /apps /scratch /lustre /work /projects"]),
        ]

        return "\n".join(str(x) for x in instructions)



import os

from hpccm.primitives import comment, copy, environment, raw, shell
from hpccm.building_blocks import packages

print(f"Evaluating {filename}")

# Add cli arguments to args_parser
parse_g = parser.add_argument_group(filename)

parse_g.add_argument(
    "--pyvista-gpu",
    dest="pyvista_gpu",
    action="store_true",
    help="Enables PyVista with hardware rendering. (Requires NVidia GPU on the host with exactly the same driver version as the build machine)",
)

### SET arguments, e.g:

# Parse local args
local_args = parser.parse_known_args()[0]

if not "jupyter/" in local_args.runtime_base_image:
    print(
        "The ogs_jupyter.py recipe requires a Jupyter base image for the "
        "runtime stage! E.g. --runtime_base_image jupyter/base-notebook-ubuntu"
    )
    exit(1)

img_file += f"-jupyter"
out_dir += f"/jupyter"

# Implement recipe
Stage1 += comment(f"Begin {filename}")

Stage1 += packages(
    apt=["git", "libgl1-mesa-glx"]
)  # for pip packages via git, for gmsh conda package

conda_packages = []
for package in versions["python"]["jupyter_image"]["conda_packages"]:
    conda_packages.append(package)

pip_packages = pip_requirements

vtk_rendering_backend = "osmesa"
if local_args.pyvista_gpu:
    vtk_rendering_backend = "egl"
else:
    conda_packages.append("mesalib")
conda_packages.append(f"vtk=*={vtk_rendering_backend}*")

if "notebook_requirements" in versions["python"]:
    for package in versions["python"]["notebook_requirements"]:
        pip_packages.append(package)

pip_packages.sort()
conda_packages.sort()

Stage1 += shell(
    commands=[
        f"mamba install --yes --quiet -c bioconda -c conda-forge  {' '.join(conda_packages)}",
        "mamba clean --all -f -y",
        'fix-permissions "${CONDA_DIR}"',
        'fix-permissions "/home/${NB_USER}"',
    ]
)

# Install via shell (and not hpccm pip) to install into conda environment
Stage1 += shell(
    commands=[
        f"pip  --no-cache-dir install {' '.join(pip_packages)}",
        'fix-permissions "${CONDA_DIR}"',
        'fix-permissions "/home/${NB_USER}"',
    ]
)

lab_overrides = """\\n\
{\\n\
  "jupyterlab-gitlab:drive": {\\n\
    "baseUrl": "https://gitlab.opengeosys.org",\\n\
    "defaultRepo": "ogs/ogs"\\n\
  }\\n\
}\\n\
"""

# GitLab extension config, points to OGS GitLab and ogs/ogs as default repo
Stage1 += shell(
    commands=[
        "echo $'c.GitLabConfig.url = \"https://gitlab.opengeosys.org\"\\n' >> /etc/jupyter/jupyter_server_config.py",
        "mkdir -p /opt/conda/share/jupyter/lab/settings",
        f"echo $'{lab_overrides}' > /opt/conda/share/jupyter/lab/settings/overrides.json",
    ]
)

if local_args.pyvista_gpu:
    # NVidia driver
    Stage1 += packages(
        apt=[
            "kmod",
            "libglvnd-dev",
            "pkg-config",
            "libxrender1",
        ]
    )

    nv_driver_version = (
        os.popen("nvidia-smi --query-gpu=driver_version --format=csv,noheader")
        .read()
        .strip()
    )

    Stage1 += shell(
        commands=[
            f"wget https://us.download.nvidia.com/XFree86/Linux-x86_64/{nv_driver_version}/NVIDIA-Linux-x86_64-{nv_driver_version}.run",
            f"sh NVIDIA-Linux-x86_64-{nv_driver_version}.run -s --no-kernel-module",
            f"rm NVIDIA-Linux-x86_64-{nv_driver_version}.run",
        ]
    )

Stage1 += comment(f"--- End {filename} ---")



import os
import subprocess
import sys
import yaml


class deployer(object):
    def __init__(self, args_deploy, cwd, image_file, **kwargs):
        deploy_config_filename = f"{cwd}/config/deploy_hosts.yml"
        if not os.path.isfile(deploy_config_filename):
            print(
                f"ERROR: {deploy_config_filename} not found but required for deploying!"
            )
            exit(1)

        with open(deploy_config_filename, "r") as ymlfile:
            deploy_config = yaml.load(ymlfile, Loader=yaml.FullLoader)
        if not args_deploy == "ALL" and args_deploy not in deploy_config:
            print(f'ERROR: Deploy host "{args_deploy}" not found in config!')
            exit(1)
        deploy_hosts = {}
        if args_deploy == "ALL":
            deploy_hosts = deploy_config
        else:
            deploy_hosts[args_deploy] = deploy_config[args_deploy]
        for deploy_host in deploy_hosts:
            deploy_info = deploy_hosts[deploy_host]
            print(f"Deploying to {deploy_info} ...")
            proxy_cmd = ""
            user_cmd = ""
            if "user" in deploy_info:
                user_cmd = f"{deploy_info['user']}@"
            if "proxy" in deploy_info:
                proxy_cmd = f"-e 'ssh -A -J {user_cmd}{deploy_info['proxy']}'"
                print(proxy_cmd)
            print(
                subprocess.check_output(
                    f"rsync -c -v {proxy_cmd} {image_file} {user_cmd}{deploy_info['host']}:{deploy_info['dest_dir']}",
                    shell=True,
                ).decode(sys.stdout.encoding)
            )



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""Package manager easybuild building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

from ogscm.building_blocks import lmod
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.shell import shell


class pm_easybuild(object):
    """Package manager easybuild building block"""

    def __init__(self, **kwargs):
        """Initialize building block"""

        # Trouble getting MRO with kwargs working correctly, so just call
        # the parent class constructors manually for now.
        # super(python, self).__init__(**kwargs)

        self.__ospackages = kwargs.get("ospackages", [])

        self.__commands = []  # Filled in by __setup()
        self.__wd = "/var/tmp"  # working directory

        self.__setup()

    def __str__(self):
        """String representation of the building block"""
        instructions = [
            comment(__doc__, reformat=False),
            packages(ospackages=self.__ospackages),
            packages(yum=["xz"], apt=["xz-utils"]),
            lmod(version="7.8.6"),
            shell(commands=self.__commands),
            environment(
                variables={
                    "MODULEPATH": "/opt/easybuild/modules/all:/home/easybuild/.local/"
                    "easybuild/modules/all:$MODULEPATH",
                    "FORCE_UNSAFE_CONFIGURE": "1",
                    # https://github.com/docker-library/python/blob/edde349541e11f66dcc79cde1674317d065ddbdd/3.6/Dockerfile#L8
                    "LANG": "C.UTF-8",
                }
            ),
            label(metadata={"PACKAGE_MANAGER": "easybuild"}),
        ]
        # Without the FORCE_UNSAFE_CONFIGURE env var some spack package
        # installations may fail due to running as root.

        return "\n".join(str(x) for x in instructions)

    def __setup(self):
        self.__ospackages.extend(
            [
                "build-essential",
                "bzip2",
                "file",
                "git",
                "gzip",
                "libssl-dev",
                "libtool",
                "make",
                "openssh-client",
                "patch",
                "python-pip",
                "python-setuptools",
                "rsh-client",
                "tar",
                "wget",
                "unzip",
            ]
        )
        self.__commands.extend(
            [
                "useradd -m easybuild",
                "mkdir -p /opt/easybuild",
                "chown easybuild:easybuild /opt/easybuild",
                "easy_install easybuild==3.7.0",
            ]
        )

    def runtime(self, _from="0"):
        """Install the runtime from a full build in a previous stage.  In this
        case there is no difference between the runtime and the
        full build."""
        return str(self)

    def install(self, ospackages=None, configs=None):
        instructions = []
        if ospackages:
            instructions.append(packages(ospackages=ospackages))
        commands = []
        for config in configs:
            commands.append(
                'runuser easybuild -l -c "eb {} -r --installpath /opt/easybuild"'.format(
                    config
                )
            )
        instructions.append(shell(commands=commands))

        return "\n".join(str(x) for x in instructions)



import subprocess
import sys
import re
import os
import platform


class builder(object):
    def __init__(
        self, args, images_out_dir, img_file, definition_file_path, tag, cwd, **kwargs
    ):
        self.__format = args.format
        self.__args = args
        self.__images_out_dir = images_out_dir
        self.__img_file = img_file
        self.__definition_file_path = definition_file_path
        self.__tag = tag
        self.__cwd = cwd

    def build(self):
        if self.__format == "singularity":
            self.build_singularity()
        else:
            self.build_docker()

    def build_singularity(self):
        sif_file = f"{self.__images_out_dir}/{self.__img_file}.sif"
        subprocess.run(
            f"sudo `which singularity` build --force {sif_file} "
            f"{self.__definition_file_path}",
            shell=True,
            check=True,
        )
        subprocess.run(f"sudo chown $USER {sif_file}", shell=True, check=True)
        print(f"Built Singularity image file: {sif_file}")
        # TODO: adapt this
        exit(0)

    def build_docker(self):
        buildx = False
        if "arm64" in platform.platform():
            buildx = True
        build_cmd = (
            f"DOCKER_BUILDKIT=1 docker{' buildx' if buildx else ''} build{' --output type=docker' if buildx else ''} {self.__args.build_args} --platform linux/amd64 "
            f"-t {self.__tag} -f {self.__definition_file_path} ."
        )
        print(f"Running: {build_cmd}")
        subprocess.run(build_cmd, shell=True, check=True)
        inspect_out = subprocess.check_output(
            f"docker inspect {self.__tag} | grep Id", shell=True
        ).decode(sys.stdout.encoding)
        image_id = re.search("sha256:(\w*)", inspect_out).group(1)
        image_id_short = image_id[0:12]

        if self.__args.upload:
            subprocess.run(f"docker push {self.__tag}", shell=True, check=True)
        image_base_name = f"{self.__images_out_dir}/{self.__img_file}-{image_id_short}"
        if self.__args.sif_file:
            self.image_file = f"{self.__images_out_dir}/{self.__args.sif_file}"
        else:
            self.image_file = f"{image_base_name}.sif"
        if self.__args.convert:
            if (
                not os.path.exists(self.image_file)
                or self.__args.force
                or self.__args.sif_file
            ):
                subprocess.run(
                    f"cd {self.__cwd} && singularity build --force {self.image_file} docker-daemon:{self.__tag}",
                    shell=True,
                    check=True,
                )
                print(f"Built Singularity image file: {self.image_file}")
            else:
                print(f"Already existing Singularity image file: {self.image_file}")

        if self.__args.enroot_file:
            self.image_file = f"{self.__images_out_dir}/{self.__args.enroot_file}"
        else:
            self.image_file = f"{image_base_name}.sqsh"
        if self.__args.convert_enroot and (
            not os.path.exists(self.image_file) or self.__args.force
        ):
            subprocess.run(
                # See https://www.mankier.com/1/mksquashfs for options.
                f"cd {self.__cwd} && rm -f {self.image_file} && ENROOT_SQUASH_OPTIONS='-comp xz -b 512K' enroot import -o {self.image_file} dockerd://{self.__tag}",
                shell=True,
                check=True,
            )
            print(f"Wrote image file {self.image_file}")

        bundle_file = f"{self.image_file[:-5]}.run"
        if self.__args.enroot_bundle and (
            not os.path.exists(bundle_file) or self.__args.force
        ):
            subprocess.run(
                f"cd {self.__cwd} && rm -f {bundle_file} && enroot bundle -o {bundle_file} {self.image_file}",
                shell=True,
                check=True,
            )
            print(f"Wrote bundle file {bundle_file}")



import hpccm
from hpccm import linux_distro
from hpccm.building_blocks import generic_cmake, packages, llvm, gnu
from hpccm.primitives import comment

print(f"Evaluating {filename}")

# Add cli arguments to args_parser
parse_g = parser.add_argument_group(filename)
parse_g.add_argument(
    "--compiler",
    type=str,
    default="gcc",
    help="The compiler to use. Possible options: off, gcc, clang",
)
parse_g.add_argument(
    "--compiler_version", type=str, default="", help="Compiler version."
)
parse_g.add_argument(
    "--fortran",
    dest="fortran",
    action="store_true",
    help="Install fortran compiler.",
)
parse_g.add_argument(
    "--iwyy",
    dest="iwyy",
    action="store_true",
    help="Install include-what-you-use (requires clang compiler)",
)

# Parse local args
local_args = parser.parse_known_args()[0]

if local_args.compiler_version == "":
    local_args.compiler_version = None  # Use default

# set image file name
compiler_version_string = local_args.compiler_version
if compiler_version_string == None:
    compiler_version_string = "default"

img_file += f"-{local_args.compiler}-{compiler_version_string}"

# Optionally set out_dir
out_dir += f"/{local_args.compiler}/{compiler_version_string}"

# Implement recipe
Stage0 += comment(f"--- Begin {filename} ---")
if local_args.compiler == "clang":
    compiler = llvm(
        extra_repository=True,
        extra_tools=True,
        version=local_args.compiler_version,
    )
else:
    compiler = gnu(
        fortran=local_args.fortran,
        version=local_args.compiler_version,
    )
toolchain = compiler.toolchain
Stage0 += compiler
# Upgrade stdc++ lib after installing new compiler
# https://stackoverflow.com/a/46613656/80480
if local_args.compiler == "gcc" and local_args.compiler_version is not None:
    Stage0 += packages(apt=["libstdc++6"])

if local_args.iwyy and local_args.compiler != "clang":
    print("--iwyy can only be used with --compiler clang")
    exit(1)
if local_args.iwyy:
    Stage0 += packages(
        ospackages=[
            "libncurses5-dev",
            "zlib1g-dev",
            f"llvm-{local_args.compiler_version}-dev",
            f"libclang-{local_args.compiler_version}-dev",
        ]
    )
    Stage0 += generic_cmake(
        cmake_opts=[
            f"-D IWYU_LLVM_ROOT_PATH=/usr/lib/llvm-{local_args.compiler_version}"
        ],
        devel_environment={"PATH": "/usr/local/iwyy/bin:$PATH"},
        directory=f"include-what-you-use-clang_{local_args.compiler_version}.0",
        prefix="/usr/local/iwyy",
        runtime_environment={"PATH": "/usr/local/iwyy/bin:$PATH"},
        url="https://github.com/include-what-you-use/include-what-"
        f"you-use/archive/clang_{local_args.compiler_version}.0.tar.gz",
    )

Stage0 += comment(f"--- End {filename} ---")



from copy import copy

import json
import math
import multiprocessing
import re
import requests

from ogscm.building_blocks.ogs_base import ogs_base
from hpccm.building_blocks import (
    boost,
    cmake,
    generic_autotools,
    generic_cmake,
    hdf5,
    packages,
    pip,
)

import hpccm
from ogscm.building_blocks.paraview import paraview
from ogscm.building_blocks.ccache import ccache
from hpccm.primitives import comment, copy, environment, raw, shell
from hpccm import linux_distro
import os
from ogscm.building_blocks.ogs import ogs
import subprocess
import hashlib

print(f"Evaluating {filename}")

# Add cli arguments to args_parser
parse_g = parser.add_argument_group(filename)
parse_g.add_argument(
    "--pm",
    type=str,
    choices=["system", "off"],
    default="system",
    help="Package manager to install third-party " "dependencies",
)
parse_g.add_argument(
    "--ogs",
    type=str,
    default="ogs/ogs@master",
    help="OGS repo on gitlab.opengeosys.org in the form 'user/repo@branch' "
    "OR 'user/repo@@commit' to checkout a specific commit "
    "OR a path to a local subdirectory to the git cloned OGS sources "
    "OR 'off' to disable OGS building "
    "OR 'clean' to disable OGS and all its dev dependencies",
)
parse_g.add_argument(
    "--cmake_args",
    type=str,
    default="",
    help="CMake argument set has to be quoted and **must**"
    " start with a space. e.g. --cmake_args ' -DFIRST="
    "TRUE -DFOO=BAR'",
)
parse_g.add_argument(
    "--cmake_preset",
    type=str,
    default="release",
    help="A CMake configuration preset to use.",
)
parse_g.add_argument(
    "--cmake_preset_file",
    type=str,
    default=None,
    help="A CMake (user) presets file as a local file path.",
)
parse_g.add_argument(
    "--ccache",
    dest="ccache",
    action="store_true",
    help="Enables ccache build caching. (Docker-only)",
)
parse_g.add_argument(
    "--cpmcache",
    dest="cpmcache",
    action="store_true",
    help="Enables CPM source caching. (Docker-only)",
)
parse_g.add_argument(
    "--parallel",
    "-j",
    type=str,
    default=math.ceil(multiprocessing.cpu_count() / 2),
    help="The number of cores to use for compilation.",
)
parse_g.add_argument(
    "--gui",
    dest="gui",
    action="store_true",
    help="Builds the GUI (Data Explorer)",
)
parse_g.add_argument(
    "--docs",
    dest="docs",
    action="store_true",
    help="Setup documentation requirements (Doxygen)",
)
parse_g.add_argument(
    "--cvode",
    dest="cvode",
    action="store_true",
    help="Install and configure with cvode",
)
parse_g.add_argument(
    "--cppcheck", dest="cppcheck", action="store_true", help="Install cppcheck"
)
parse_g.add_argument("--gcovr", dest="gcovr", action="store_true", help="Install gcovr")
parse_g.add_argument(
    "--mfront",
    dest="mfront",
    action="store_true",
    help="Install tfel and build OGS with -DOGS_USE_MFRONT=ON",
)
parse_g.add_argument(
    "--insitu",
    dest="insitu",
    action="store_true",
    help="Builds with insitu capabilities",
)
parse_g.add_argument(
    "--dev",
    dest="dev",
    action="store_true",
    help="Installs development tools (vim, gdb)",
)
parse_g.add_argument(
    "--mkl",
    dest="mkl",
    action="store_true",
    help="Use MKL. By setting this option, you agree to the [Intel End User License Agreement](https://software.intel.com/en-us/articles/end-user-license-agreement).",
)
parse_g.add_argument(
    "--netcdf",
    dest="netcdf",
    action="store_true",
    help="Install netcdf and build OGS with -DOGS_USE_NETCDF=ON",
)
if toolchain.CC == "mpicc":
    parse_g.add_argument(
        "--petsc_configure_args",
        type=str,
        default="--with-fc=0",
        help="PETSc configuration arguments; has to be quoted.",
    )
parse_g.add_argument("--version_file", type=str, help="OGS versions.json file")
parse_g.add_argument(
    "--boost-sourceforge",
    dest="boost_sourceforge",
    action="store_true",
    help="Boolean flag to specify whether Boost should be downloaded from SourceForge rather than the current Boost repository.",
)
parse_g.add_argument(
    "--keep-ogs-source",
    dest="keep_source",
    action="store_true",
    help="Boolean flag to specify whether the OGS source directory should be preserved.",
)
parse_g.add_argument(
    "--keep-ogs-build",
    dest="keep_build",
    action="store_true",
    help="Boolean flag to specify whether the OGS build directory should be preserved.",
)
parse_g.add_argument(
    "--run-ctest",
    dest="run_ctest",
    action="store_true",
    help="Boolean flag to specify whether to run the OGS ctest-target.",
)

# Parse local args
local_args = parser.parse_known_args()[0]

branch_is_release = False
git_version = ""
name_start = ""
repo = None

if local_args.ogs not in ["off", "clean"]:  # != "off" and local_args.ogs != "clean":
    if os.path.isdir(local_args.ogs):
        if local_args.run_ctest and args.format == "docker":
            print("`--run_ctest` can not be used with `--ogs local/path`")
            exit(1)
        repo = "local"
        commit_hash = subprocess.run(
            ["cd {} && git rev-parse HEAD".format(local_args.ogs)],
            capture_output=True,
            text=True,
            shell=True,
        ).stdout.rstrip()
        with open(f"{local_args.ogs}/web/data/versions.json") as fp:
            versions = json.load(fp)
        with open(f"{local_args.ogs}/Tests/Data/requirements.txt") as fp:
            pip_requirements = fp.read().splitlines()
        if "GITLAB_CI" in os.environ:
            if "CI_COMMIT_TAG" in os.environ:
                branch = "master"
            elif "CI_COMMIT_BRANCH" in os.environ:
                branch = os.environ["CI_COMMIT_BRANCH"]
            elif "CI_MERGE_REQUEST_SOURCE_BRANCH_NAME" in os.environ:
                branch = os.environ["CI_MERGE_REQUEST_SOURCE_BRANCH_NAME"]
            if "OGS_VERSION" in os.environ:
                git_version = os.environ["OGS_VERSION"]
        else:
            branch = subprocess.run(
                [
                    "cd {} && git branch | grep \* | cut -d ' ' -f2".format(
                        local_args.ogs
                    )
                ],
                capture_output=True,
                text=True,
                shell=True,
            ).stdout
            git_version = subprocess.run(
                ["cd {} && git describe --tags".format(local_args.ogs)],
                capture_output=True,
                text=True,
                shell=True,
            ).stdout.strip()
    else:
        # Get git commit hash and construct image tag name
        repo, branch, *commit = local_args.ogs.split("@")
        if commit:
            commit_hash = commit[0]
            if branch == "":
                branch = "master"
            versions = json.loads(
                requests.get(
                    f"https://gitlab.opengeosys.org/{repo}/-/raw/{commit_hash}/web/data/versions.json"
                ).text
            )
            pip_requirements = requests.get(
                f"https://gitlab.opengeosys.org/{repo}/-/raw/{commit_hash}/Tests/Data/requirements.txt"
            ).text.splitlines()
        else:
            if re.search(r"[\d.]+", branch):
                branch_is_release = True
            repo_split = repo.split("/")
            response = requests.get(
                f"https://gitlab.opengeosys.org/api/v4/projects/{repo.replace('/', '%2F')}/repository/commits?ref_name={branch}"
            )
            response_data = json.loads(response.text)
            commit_hash = response_data[0]["id"]
            # ogs_tag = args.ogs.replace('/', '.').replace('@', '.')
            versions = json.loads(
                requests.get(
                    f"https://gitlab.opengeosys.org/{repo}/-/raw/{branch}/web/data/versions.json"
                ).text
            )
            pip_requirements = requests.get(
                f"https://gitlab.opengeosys.org/{repo}/-/raw/{branch}/Tests/Data/requirements.txt"
            ).text.splitlines()

        if branch_is_release:
            name_start = f"ogs-{branch}"
        else:
            name_start = f"ogs-{commit_hash[:8]}"

if local_args.version_file:
    with open(local_args.version_file) as fp:
        versions = json.load(fp)
if versions == None:
    versions = json.loads(
        requests.get(
            f"https://gitlab.opengeosys.org/ogs/ogs/-/raw/master/web/data/versions.json"
        ).text
    )

if local_args.cmake_preset_file:
    # Make path absolute
    local_args.cmake_preset_file = os.path.abspath(
        os.path.expanduser(os.path.expandvars(local_args.cmake_preset_file))
    )

folder = f"/{name_start}/{local_args.pm}".replace("//", "/")

if len(local_args.cmake_args) > 0:
    cmake_args_hash = hashlib.md5(
        " ".join(local_args.cmake_args).encode("utf-8")
    ).hexdigest()
    cmake_args_hash_short = cmake_args_hash[:8]
    folder += f"/cmake-{cmake_args_hash_short}"

# set image file name
img_file += folder.replace("/", "-")

if local_args.gui:
    img_file += "-gui"
if local_args.ogs != "off" and not args.runtime_only:
    img_file += "-dev"

# Optionally set out_dir
out_dir += folder

# Implement recipe
Stage0 += comment(f"--- Begin {filename} ---")

cmake_args = local_args.cmake_args.strip().split(" ")

Stage0 += ogs_base()
if local_args.gui:
    Stage0 += packages(
        apt=[
            "mesa-common-dev",
            "libgl1-mesa-dev",
            "libglu1-mesa-dev",
            "libxt-dev",
            "libglib2.0-0",
        ],
        yum=[
            "mesa-libOSMesa-devel",
            "mesa-libGL-devel",
            "mesa-libGLU-devel",
            "libXt-devel",
        ],
    )
    Stage1 += packages(
        apt=[
            "libosmesa6",
            "libgl1-mesa-glx",
            "libglu1-mesa",
            "libxt6",
            "libopengl0",
        ],
        yum=["mesa-libOSMesa", "mesa-libGL", "mesa-libGLU", "libXt"],
    )

versions_master = json.loads(
    requests.get(
        f"https://gitlab.opengeosys.org/ogs/ogs/-/raw/master/web/data/versions.json?inline=false"
    ).text
)

if local_args.mkl:
    Stage0 += packages(ospackages=["ca-certificates", "gnupg", "wget"])
    Stage1 += packages(ospackages=["ca-certificates", "gnupg", "wget"])
    mkl_version = "2021.4.0"
    Stage0 += packages(
        apt_keys=[
            "https://apt.repos.intel.com/intel-gpg-keys/GPG-PUB-KEY-INTEL-SW-PRODUCTS-2023.PUB"
        ],
        apt_repositories=["deb https://apt.repos.intel.com/oneapi all main"],
        ospackages=[f"intel-oneapi-mkl-devel-{mkl_version}"],
    )
    Stage1 += packages(
        apt_keys=[
            "https://apt.repos.intel.com/intel-gpg-keys/GPG-PUB-KEY-INTEL-SW-PRODUCTS-2023.PUB"
        ],
        apt_repositories=["deb https://apt.repos.intel.com/oneapi all main"],
        ospackages=[f"intel-oneapi-mkl-{mkl_version}"],
    )
    mkl_env = environment(
        variables={
            "LD_LIBRARY_PATH": f"/opt/intel/oneapi/tbb/{mkl_version}/env/../lib/intel64/gcc4.8:/opt/intel/oneapi/mkl/{mkl_version}/lib/intel64:/opt/intel/oneapi/compiler/{mkl_version}/linux/lib:/opt/intel/oneapi/compiler/{mkl_version}/linux/lib/x64:/opt/intel/oneapi/compiler/{mkl_version}/linux/lib/emu:/opt/intel/oneapi/compiler/{mkl_version}/linux/compiler/lib/intel64_lin:$LD_LIBRARY_PATH"
        }
    )
    Stage0 += mkl_env
    Stage1 += mkl_env
    cmake_args.append("-DOGS_USE_MKL=ON")

if local_args.ogs != "clean":
    cmake_version = versions_master["tested_version"]["cmake"]
    if "cmake" in versions["tested_version"]:
        cmake_version = versions["tested_version"]["cmake"]
    Stage0 += cmake(eula=True, version=cmake_version)
    if local_args.pm == "system":
        if local_args.mfront:
            # for mfront python bindings
            boost_bootsrap_opts = ["--with-python=python3"]
            if toolchain.CC == "clang":
                boost_bootsrap_opts.append("--with-toolset=clang")
            Stage0 += boost(
                b2_opts=["headers"],
                bootstrap_opts=boost_bootsrap_opts,
                ldconfig=True,
                sourceforge=local_args.boost_sourceforge,
                version="1.78.0",  # versions["minimum_version"]["boost"], # is too old for gcc 12
            )
            Stage0 += environment(variables={"BOOST_ROOT": "/usr/local/boost"})
        Stage0 += packages(
            apt=["libxml2-dev", "xsltproc"], yum=["libxml2-devel", "libxslt"]
        )
        vtk_cmake_args = []
        if "libraries" in versions:
            for option in versions["libraries"]["vtk"]["options"]:
                if eval(option["condition"]["ogscm"]):
                    for cmake_option in option["cmake"]:
                        vtk_cmake_args.append(f"-D{cmake_option}")
            # Reverse so the VTK_GROUP_ vars are at the end of the list, otherwise they have no effect.
            vtk_cmake_args = vtk_cmake_args[::-1]
        else:
            vtk_cmake_args = [
                "-DVTK_MODULE_ENABLE_VTK_IOXML=YES",
                "-DVTK_MODULE_ENABLE_VTK_IOLegacy=YES",
                "-DVTK_GROUP_ENABLE_Rendering=DONT_WANT",
                "-DVTK_GROUP_ENABLE_StandAlone=DONT_WANT",
            ]
            if local_args.gui:
                vtk_cmake_args = [
                    "-DVTK_BUILD_QT_DESIGNER_PLUGIN=OFF",
                    "-DVTK_Group_Qt=ON",
                    "-DVTK_QT_VERSION=5",
                ]
            if toolchain.CC == "mpicc":
                vtk_cmake_args.extend(
                    [
                        "-DVTK_MODULE_ENABLE_VTK_IOParallelXML=YES",
                        "-DVTK_MODULE_ENABLE_VTK_ParallelMPI=YES",
                    ]
                )

        if local_args.netcdf:
            Stage0 += packages(apt=["libnetcdf-c++4-dev"])
            Stage1 += packages(apt=["libnetcdf-c++4"])

        if local_args.gui:
            Stage0 += packages(
                apt=[
                    "libgeotiff-dev",
                    "libshp-dev",
                    "libnetcdf-c++4-dev",
                ],
                yum=[
                    "libgeotiff-devel",
                    "shapelib-devel",
                    "netcdf-devel",
                ],
            )
            Stage1 += packages(
                apt=[
                    "geotiff-bin",
                    "shapelib",
                    "libnetcdf-c++4",
                    "libglib2.0-0",
                    "libdbus-1-3",
                    "libexpat1",
                    "libfontconfig1",
                    "libfreetype6",
                    "libgl1-mesa-glx",
                    "libglib2.0-0",
                    "libx11-6",
                    "libx11-xcb1",
                    "libxkbcommon-x11-0",
                ],
                # TODO: Add runtime packages for yum
                yum=[
                    "libgeotiff",
                    "shapelib",
                    "netcdf",
                ],
            )
            # TODO: will not work with clang
            qt_install_dir = "/opt/qt"
            qt_version = versions["minimum_version"]["qt"]
            qt_dir = f"{qt_install_dir}/{qt_version}/gcc_64"
            Stage0 += shell(
                commands=["pip3 install --break-system-packages aqtinstall==3.1.6"]
            )
            Stage0 += shell(
                commands=[
                    f"aqt install-qt --outputdir {qt_install_dir} linux desktop {qt_version} gcc_64",
                    f"aqt install-qt --outputdir {qt_install_dir} linux desktop {qt_version} gcc_64 --archives qtxmlpatterns qtx11extras",
                ]
            )
            Stage1 += copy(_from="0", src=qt_install_dir, dest=qt_install_dir)
            Stage0 += environment(
                variables={
                    "LD_LIBRARY_PATH": f"{qt_dir}/lib:$LD_LIBRARY_PATH",
                    "PATH": f"{qt_dir}/bin:$PATH",
                    "QTDIR": qt_dir,
                }
            )
            Stage1 += environment(
                variables={
                    "LD_LIBRARY_PATH": f"{qt_dir}/lib:$LD_LIBRARY_PATH",
                    "PATH": f"{qt_dir}/bin:$PATH",
                    "QTDIR": qt_dir,
                }
            )

        if hpccm.config.g_linux_distro == linux_distro.CENTOS:
            # otherwise linker error, maybe due to gcc 10?
            vtk_cmake_args.extend(
                [
                    "-DBUILD_SHARED_LIBS=OFF",
                    "-DCMAKE_POSITION_INDEPENDENT_CODE=ON",
                ]
            )
        if local_args.insitu:
            if local_args.gui:
                print("--gui can not be used with --insitu!")
                exit(1)
            Stage0 += paraview(
                cmake_args=["-DPARAVIEW_USE_PYTHON=ON"],
                edition="CATALYST",
                ldconfig=True,
                toolchain=toolchain,
                version="v5.10.1",
            )
        else:
            vtk_version = versions["minimum_version"]["vtk"]
            Stage0 += generic_cmake(
                cmake_opts=vtk_cmake_args,
                devel_environment={"VTK_ROOT": "/usr/local/vtk"},
                directory=f"vtk-v{vtk_version}",
                ldconfig=True,
                prefix="/usr/local/vtk",
                toolchain=toolchain,
                url=f"https://gitlab.kitware.com/vtk/vtk/-/archive/v{vtk_version}/vtk-v{vtk_version}.tar.gz",
            )
        if toolchain.CC == "mpicc":
            Stage0 += packages(yum=["diffutils"])
            if "--download-ptscotch" in local_args.petsc_configure_args:
                Stage0 += packages(ospackages=["bison", "flex"])
            petsc_version = versions["minimum_version"]["petsc"]
            petsc_args = local_args.petsc_configure_args.strip().split(" ")
            petsc_configure_opts = [
                f"CC={toolchain.CC}",
                f"CXX={toolchain.CXX}",
                f"FC={toolchain.FC}",
                f"F77={toolchain.F77}",
                f"F90={toolchain.F90}",
                "--CFLAGS='-O3'",
                "--CXXFLAGS='-O3'",
                "--FFLAGS='-O3'",
                "--with-debugging=no",
            ]
            petsc_configure_opts.extend(petsc_args)
            if "--download-f2cblaslapack" not in local_args.petsc_configure_args:
                petsc_configure_opts.extend(
                    [
                        "--download-f2cblaslapack=1",
                    ]
                )
            Stage0 += generic_autotools(
                configure_opts=petsc_configure_opts,
                devel_environment={"CMAKE_PREFIX_PATH": "/usr/local/petsc"},
                directory=f"petsc-{petsc_version}",
                ldconfig=True,
                prefix="/usr/local/petsc",
                toolchain=toolchain,
                url=f"http://ftp.mcs.anl.gov/pub/petsc/release-snapshots/petsc-lite-{petsc_version}.tar.gz",
            )
        # Needs to be after PETSc: superlu_dist does not work Ninja
        Stage0 += environment(
            variables={
                "CMAKE_GENERATOR": "Ninja",
            }
        )

        eigen_version = versions["minimum_version"]["eigen"]
        Stage0 += generic_cmake(
            devel_environment={
                "Eigen3_ROOT": "/usr/local/eigen",
                "Eigen3_DIR": "/usr/local/eigen",
            },
            directory=f"eigen-{eigen_version}",
            prefix="/usr/local/eigen",
            toolchain=toolchain,
            url=f"https://gitlab.com/libeigen/eigen/-/archive/{eigen_version}/eigen-{eigen_version}.tar.gz",
        )
        hdf5_cofigure_opts = ["--enable-cxx"]
        if toolchain.CC == "mpicc":
            hdf5_cofigure_opts = ["--enable-parallel", "--enable-shared"]
        hdf5_version = versions_master["minimum_version"]["hdf5"]
        if "hdf5" in versions["minimum_version"]:
            hdf5_version = versions["minimum_version"]["hdf5"]
        Stage0 += hdf5(
            configure_opts=hdf5_cofigure_opts,
            ldconfig=True,
            toolchain=toolchain,
            version=hdf5_version,
        )

    if local_args.pm == "off" and local_args.netcdf:
        Stage0 += packages(apt=["libnetcdf-dev"])
        Stage1 += packages(apt=["libnetcdf"])

if local_args.cvode:
    # TODO version
    Stage0 += generic_cmake(
        cmake_opts=[
            "-D EXAMPLES_INSTALL=OFF",
            "-D BUILD_SHARED_LIBS=OFF",
            "-D CMAKE_POSITION_INDEPENDENT_CODE=ON",
        ],
        devel_environment={"CVODE_ROOT": "/usr/local/cvode"},
        directory="cvode-2.8.2",
        prefix="/usr/local/cvode",
        toolchain=toolchain,
        url="https://github.com/ufz/cvode/archive/2.8.2.tar.gz",
    )

if local_args.cppcheck:
    cppcheck_version = "2.7.5"
    Stage0 += generic_cmake(
        devel_environment={"PATH": "/usr/local/cppcheck/bin:$PATH"},
        directory=f"cppcheck-{cppcheck_version}",
        prefix="/usr/local/cppcheck",
        runtime_environment={"PATH": "/usr/local/cppcheck/bin:$PATH"},
        toolchain=toolchain,
        url=f"https://github.com/danmar/cppcheck/archive/refs/tags/{cppcheck_version}.tar.gz",
    )

if local_args.docs:
    Stage0 += packages(ospackages=["graphviz", "texlive-base"])
    # doxygen package is out-dated, install Doxygen from binary
    Stage0 += shell(
        chdir="/tmp",
        commands=[
            "wget https://www.doxygen.nl/files/doxygen-1.9.6.linux.bin.tar.gz",
            "tar xf doxygen-1.9.6.linux.bin.tar.gz -C /usr/local --strip-components=1",
            "pip3 install --break-system-packages lizard",
        ],
    )
    Stage0 += packages(ospackages=["python3-pandas"])

    cpp_dep_commit = "17ea25894333813fa4798b40f82d381cefcad0b8"
    Stage0 += generic_cmake(
        directory=f"cpp-dependencies-{cpp_dep_commit}",
        cmake_opts=["-DWITH_BOOST=OFF"],
        ldconfig=True,
        url=f"https://github.com/tomtom-international/cpp-dependencies/archive/{cpp_dep_commit}.zip",
        prefix="/usr/local/cpp-dependencies",
        runtime_environment={"PATH": "/usr/local/cpp-dependencies/bin:$PATH"},
        devel_environment={"PATH": "/usr/local/cpp-dependencies/bin:$PATH"},
        toolchain=toolchain,
    )
if local_args.gcovr:
    Stage0 += shell(commands=["pip3 install --break-system-packages gcovr"])

if local_args.dev:
    Stage0 += packages(
        ospackages=["neovim", "gdb", "silversearcher-ag", "ssh-client", "less"]
    )

if local_args.mfront and local_args.pm == "system":
    tfel_version = versions_master["minimum_version"]["tfel-rliv"]
    if "tfel-rliv" in versions["minimum_version"]:
        tfel_version = versions["minimum_version"]["tfel-rliv"]
    Stage0 += generic_cmake(
        cmake_opts=["-Denable-python-bindings=ON", "-Denable-portable-build=ON"],
        directory=f"tfel-rliv-{tfel_version}",
        ldconfig=True,
        url=f"https://github.com/thelfer/tfel/archive/refs/heads/rliv-{tfel_version}.zip",
        prefix="/usr/local/tfel",
        runtime_environment={"PATH": "/usr/local/tfel/bin:$PATH"},
        devel_environment={"PATH": "/usr/local/tfel/bin:$PATH"},
        toolchain=toolchain,
    )
    tfel_env = environment(
        variables={
            "TFELHOME": "/usr/local/tfel",
            # TODO: Don't hard-code python version
            "PYTHONPATH": "/usr/local/tfel/lib/python3.8/site-packages:$PYTHONPATH",
        }
    )
    Stage0 += tfel_env
    Stage1 += tfel_env
    cmake_args.append("-DOGS_USE_MFRONT=ON")

# Used to fix RPATH in petsc.so build as external dependency
# pkg-config for PETSc finding
Stage0 += packages(ospackages=["patchelf", "pkg-config"])

if local_args.ccache:
    Stage0 += ccache(cache_size="15G")
if local_args.ogs != "off" and local_args.ogs != "clean":
    mount_args = ""
    if local_args.ccache:
        mount_args += f" --mount=type=cache,target=/opt/ccache"
    if local_args.cpmcache:
        mount_args += f" --mount=type=cache,target=/opt/cpmcache,sharing=locked"
        cmake_args.append("-DCPM_SOURCE_CACHE=/opt/cpmcache")
    if local_args.cvode:
        cmake_args.append("-DOGS_USE_CVODE=ON")
    if local_args.gui:
        cmake_args.append("-DOGS_BUILD_GUI=ON")
    if local_args.insitu:
        cmake_args.append("-DOGS_INSITU=ON")
    if local_args.netcdf:
        cmake_args.append("-DOGS_USE_NETCDF=ON")

    Stage0 += raw(docker=f"ARG OGS_COMMIT_HASH={commit_hash}")

    if repo == "local":
        print(f"chdir to {local_args.ogs}")
        os.chdir(local_args.ogs)

    Stage0 += ogs(
        repo=repo,
        branch=branch,
        commit=commit_hash,
        git_version=git_version,
        toolchain=toolchain,
        cmake_args=cmake_args,
        cmake_preset=local_args.cmake_preset,
        cmake_preset_file=local_args.cmake_preset_file,
        parallel=local_args.parallel,
        remove_build=not local_args.keep_build,
        remove_source=not local_args.keep_source,
        mount_args=mount_args,
        run_ctest=local_args.run_ctest,
    )

# Required for vtk from Python (for notebooks, VTUInterface)
# https://github.com/Kaggle/docker-python/pull/358
# xvfb for PyVista
Stage0 += packages(
    apt=["libgl1", "xvfb", "libgl1-mesa-glx", "libglu1-mesa"],
    yum=["mesa-libGL", "xorg-x11-server-Xvfb", "mesa-libGLU"],
)
Stage1 += packages(
    apt=["libgl1", "xvfb", "libgl1-mesa-glx", "libglu1-mesa"],
    yum=["mesa-libGL", "xorg-x11-server-Xvfb", "mesa-libGLU"],
)



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""Lmod building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os

from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.primitives.shell import shell
from hpccm.templates.ConfigureMake import ConfigureMake
from hpccm.templates.rm import rm
from hpccm.templates.tar import tar
from hpccm.templates.wget import wget


class lmod(ConfigureMake, rm, tar, wget):
    """Lmod building block"""

    def __init__(self, **kwargs):
        """Initialize building block"""

        # Trouble getting MRO with kwargs working correctly, so just call
        # the parent class constructors manually for now.
        # super(python, self).__init__(**kwargs)
        ConfigureMake.__init__(self, **kwargs)
        rm.__init__(self, **kwargs)
        tar.__init__(self, **kwargs)
        wget.__init__(self, **kwargs)

        self.__commands = []  # Filled in by __setup()
        self.__wd = "/var/tmp"  # working directory
        self.prefix = "/opt/apps"
        self.version = kwargs.get("version", "7.8.6")

    def __str__(self):
        """String representation of the building block"""
        tarfile = "{}.tar.gz".format(self.version)
        source = os.path.join(self.__wd, "Lmod-{}".format(self.version))
        instructions = []
        instructions.extend(
            [
                comment(__doc__, reformat=False),
                # For Doxygen diagrams and bibtex references
                packages(
                    ospackages=[
                        "liblua5.1-0",
                        "liblua5.1-0-dev",
                        "lua-filesystem-dev",
                        "lua-posix-dev",
                        "lua5.1",
                        "tclsh",
                    ]
                ),
                shell(
                    commands=[
                        self.download_step(
                            url="https://github.com/TACC/Lmod/archive/{}".format(
                                tarfile
                            ),
                            directory=self.__wd,
                        ),
                        self.untar_step(
                            tarball=os.path.join(self.__wd, tarfile),
                            directory=self.__wd,
                        ),
                        self.configure_step(directory=source),
                        self.install_step(),
                        "ln -s /opt/apps/lmod/lmod/init/profile /etc/profile.d/z00_lmod.sh",
                        "ln -s /opt/apps/lmod/lmod/init/cshrc   /etc/profile.d/z00_lmod.csh",
                    ]
                ),
                environment(
                    variables={"MODULEPATH": "/opt/easybuild/modules/all:$MODULEPATH"}
                ),
            ]
        )

        return "\n".join(str(x) for x in instructions)

    def runtime(self, _from="0"):
        """Install the runtime from a full build in a previous stage.  In this
        case there is no difference between the runtime and the
        full build."""
        return str(self)



# Recipe template
# Have a look and change statements with ### SET comment

### SET imports
from hpccm.primitives import comment
from hpccm.building_blocks import xx

print(f"Evaluating {filename}")

# Add cli arguments to args_parser
parse_g = parser.add_argument_group(filename)

### SET arguments, e.g:
parse_g.add_argument("--my_arg", type=str, default="default_value")

# Parse local args
local_args = parser.parse_known_args()[0]

### SET append to image file name, e.g.:
img_file += f"-someName-{local_args.my_arg}"

### SET Append to out_dir, e.g.:
out_dir += f"/someName/{local_args.my_arg}"

# Implement recipe
Stage0 += comment(f"Begin {filename}")

Stage0 += xx(arg="value")

Stage0 += comment(f"--- End {filename} ---")



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""paraview building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import re
import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.generic_cmake import generic_cmake
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.copy import copy
from hpccm.primitives.environment import environment
from hpccm.toolchain import toolchain
from hpccm.primitives.shell import shell
from hpccm.common import linux_distro


class paraview(
    bb_base,
    hpccm.templates.CMakeBuild,
    hpccm.templates.ldconfig,
    hpccm.templates.rm,
    hpccm.templates.tar,
    hpccm.templates.wget,
):
    """The `paraview` building block downloads and installs the
    [paraview](https://paraview.org/) component.

    # Parameters

    prefix: The top level installation location.  The default value
    is `/usr/local/paraview`.

    version: The version of paraview source to download.  The default
    value is `master`.

    # Examples

    ```python
    paraview()
    ```

    """

    def __init__(self, **kwargs):
        super(paraview, self).__init__(**kwargs)

        self.__cmake_args = kwargs.get("cmake_args", [])
        self.__edition = kwargs.get("edition", "CANONICAL")
        self.__ospackages = kwargs.get("ospackages", [])
        self.__parallel = kwargs.get("parallel", "$(nproc)")
        self.__prefix = kwargs.get("prefix", "/usr/local/paraview")
        self.__shared = kwargs.get("shared", True)
        self.__toolchain = kwargs.get("toolchain", toolchain())
        self.__version = kwargs.get("version", "master")
        self.__modules = kwargs.get("modules", [])

        # TODO:
        if False:
            match = re.match(
                r"(?P<major>\d+)\.(?P<minor>\d+)\.(?P<revision>\d+)", self.__version
            )
            short_version = "{0}.{1}".format(
                match.groupdict()["major"], match.groupdict()["minor"]
            )
            self.__baseurl = kwargs.get(
                "baseurl",
                "https://www.paraview.org/files/release/{0}".format(short_version),
            )
        self.__environment_variables = {}

        self.__instructions()

    def __instructions(self):
        self += comment("paraview {}".format(self.__version))

        if hpccm.config.g_linux_distro == linux_distro.CENTOS:
            dist = "rpm"
            self += shell(
                commands=[
                    "wget https://github.com/ninja-build/ninja/releases/download/v1.10.0/ninja-linux.zip",
                    "unzip ninja-linux.zip",
                    "mv ninja /usr/local/bin",
                    "rm ninja-linux.zip",
                ]
            )
        else:
            self.__ospackages.extend(["ninja-build"])

        self.__cmake_args.extend(
            [
                "-G Ninja",
                "-DCMAKE_BUILD_TYPE=Release",
                "-DPARAVIEW_BUILD_EDITION={}".format(self.__edition),
            ]
        )
        if not self.__shared:
            self.__cmake_args.append("-DBUILD_SHARED_LIBS=OFF")
        if self.__toolchain.CC == "mpicc":
            self.__cmake_args.append("-DPARAVIEW_USE_MPI=ON")
        for module in self.__modules:
            self.__cmake_args.append("-DVTK_MODULE_ENABLE_{}=YES".format(module))

        # TODO: Install dependencies for rendering editions (ospackages)

        self += generic_cmake(
            branch=self.__version,
            cmake_opts=self.__cmake_args,
            prefix=self.__prefix,
            toolchain=self.__toolchain,
            recursive=True,
            repository="https://gitlab.kitware.com/paraview/paraview.git",
        )
        self.__environment_variables["ParaView_DIR"] = self.__prefix
        # Set library path
        if self.__shared:
            libpath = os.path.join(self.__prefix, "lib")
            if hpccm.config.g_linux_distro == linux_distro.CENTOS:
                libpath += "64"
            if self.ldconfig:
                self += shell(commands=[self.ldcache_step(directory=libpath)])
            else:
                self.__environment_variables["LD_LIBRARY_PATH"] = (
                    "{}:$LD_LIBRARY_PATH".format(libpath)
                )

        self += environment(variables=self.__environment_variables)

    def runtime(self, _from="0"):
        if not self.__shared:
            return str(comment("ParaView (empty)"))
        instructions = []
        instructions.append(comment("ParaView {}".format(self.__version)))
        instructions.append(copy(_from=_from, src=self.__prefix, dest=self.__prefix))
        if self.ldconfig:
            libpath = os.path.join(self.__prefix, "lib")
            if hpccm.config.g_linux_distro == linux_distro.CENTOS:
                libpath += "64"
            instructions.append(shell(commands=[self.ldcache_step(directory=libpath)]))

        instructions.append(environment(variables=self.__environment_variables))
        return "\n".join(str(x) for x in instructions)



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""ccache building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.shell import shell


class ccache(bb_base):
    """ccache building block"""

    def __init__(self, **kwargs):
        """Initialize building block"""
        super(ccache, self).__init__()
        self.__cache_dir = kwargs.get("cache_dir", "/opt/ccache")
        self.__cache_size = kwargs.get("cache_size", "5G")

        self.__instructions()

    def __instructions(self):
        self += comment(__doc__, reformat=False)
        self += packages(ospackages=["ccache"])
        self += shell(
            commands=["mkdir -p {0} && chmod 777 {0}".format(self.__cache_dir)]
        )
        self += environment(
            variables={
                "CCACHE_DIR": self.__cache_dir,
                "CCACHE_MAXSIZE": self.__cache_size,
                "CCACHE_SLOPPINESS": "pch_defines,time_macros",
            }
        )
        self += label(
            metadata={"ccache.dir": self.__cache_dir, "ccache.size": self.__cache_size}
        )

    # No runtime



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""OGS building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os
import re
import subprocess

import hpccm.templates.rm

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.copy import copy
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.runscript import runscript
from hpccm.primitives.shell import shell
from hpccm.toolchain import toolchain

import ogscm
from ogscm.config import package_manager


class ogs(bb_base, hpccm.templates.CMakeBuild, hpccm.templates.rm):
    """OGS building block"""

    def __init__(self, **kwargs):
        """Initialize building block"""
        super(ogs, self).__init__(**kwargs)

        self.__cmake_args = kwargs.get("cmake_args", [])
        self.__cmake_preset = kwargs.get("cmake_preset", "release")
        self.__cmake_preset_file = kwargs.get("cmake_preset_file", None)
        self.__ospackages = []
        self.__parallel = kwargs.get("parallel", 4)
        self.__prefix = kwargs.get("prefix", "/usr/local/ogs")
        self.__remove_dev = kwargs.get("remove_dev", False)
        self.__remove_build = kwargs.get("remove_build", False)
        self.__remove_source = kwargs.get("remove_source", False)
        self.__shared = kwargs.get("shared", True)
        self.__skip_clone = kwargs.get("skip_clone", False)
        self.__toolchain = kwargs.get("toolchain", toolchain())
        self.__repo = kwargs.get("repo", "ufz/ogs.git")
        self.__branch = kwargs.get("branch", "master")
        self.__commit = kwargs.get("commit")
        self.__git_version = kwargs.get("git_version")
        self.__shell_args = kwargs.get("mount_args", "")
        self.__run_ctest = kwargs.get("run_ctest", False)

        if self.__repo == "local":
            self.__skip_clone = True
            self.__remove_source = False
            self.__shell_args += " --mount=type=bind,target={}/src,rw".format(
                self.__prefix
            )

        # Filled in by __setup():
        self.__commands = []
        self.__environment_variables = {}
        self.__labels = {}

        self.__setup()

        # Fill in container instructions
        self.__instructions()

    def __instructions(self):
        self += comment(
            "OpenGeoSys build from repo {0}, branch {1}".format(
                self.__repo, self.__branch
            )
        )
        self += packages(ospackages=self.__ospackages)
        self += shell(
            commands=self.__commands,
            _arguments=self.__shell_args,
        )

        if self.__environment_variables:
            self += environment(variables=self.__environment_variables)
        if self.__labels:
            self += label(metadata=self.__labels)

    def __setup(self):
        """Construct the series of shell commands, i.e., fill in
        self.__commands"""
        # Get the source
        if self.__skip_clone:
            self.__cmake_args.append("-DOGS_VERSION={}".format(self.__git_version))
        else:
            self.__commands.extend(
                [
                    "mkdir -p {0} && cd {0}".format(self.__prefix),
                    "git clone --filter=blob:limit=100k --branch {} https://gitlab.opengeosys.org/{} src".format(
                        self.__branch, self.__repo
                    ),
                ]
            )
            if self.__commit:
                self.__commands.append(
                    "(cd src && git reset --hard {})".format(self.__commit)
                )
            self.__commands.append("(cd src && git fetch --tags)")
        if self.__cmake_preset_file:
            f = open(self.__cmake_preset_file, "r")
            contents = f.read()
            contents = contents.replace("\n", "\\n")
            self.__commands.append(
                f"echo '{contents}' >> /usr/local/ogs/src/CMakeUserPresets.json"
            )

        # Default CMake arguments
        self.__cmake_args.extend(
            [
                "-G Ninja",
                "-DCMAKE_INSTALL_PREFIX={}".format(self.__prefix),
                "-DOGS_CPU_ARCHITECTURE=OFF",  # optimization handled via --cpu-target argument
                "-DOGS_INSTALL_DEPENDENCIES=ON",
            ]
        )

        self.__cmake_args.append(
            "-DBUILD_SHARED_LIBS={}".format("ON" if self.__shared else "OFF")
        )
        if self.__toolchain.CC == "mpicc":
            self.__cmake_args.append("-DOGS_USE_PETSC=ON")

        build_directory = "{}/build".format(self.__prefix)
        self.__cmake_args.extend(
            [f"--preset={self.__cmake_preset}", "-B", build_directory]
        )

        # Configure and build
        if "-DOGS_USE_MKL=ON" in self.__cmake_args:
            self.__commands.append(". /opt/intel/oneapi/setvars.sh")
        self.__commands.append(
            self.configure_step(
                directory="{}/src".format(self.__prefix),
                build_directory=build_directory,
                opts=self.__cmake_args,
                toolchain=self.__toolchain,
            )
        )

        # Install only when build dir is removed
        self.__commands.append(
            self.build_step(
                target=("install" if self.__remove_build else "all"),
                parallel=self.__parallel,
            )
        )

        # ctest
        if self.__run_ctest:
            self.__commands.append(self.build_step(target="ctest"))

        # Cleanup
        bin_dir = f"{self.__prefix}/build/bin"
        if self.__remove_build:
            self.__commands.append(
                self.cleanup_step(items=[os.path.join(self.__prefix, "build")])
            )
            bin_dir = f"{self.__prefix}/bin"
        if self.__remove_source:
            self.__commands.append(
                self.cleanup_step(items=[os.path.join(self.__prefix, "src")])
            )

        # Environment
        self.__environment_variables["PATH"] = f"{bin_dir}:$PATH"

        # Labels
        if self.__repo == "local":
            self.__labels["version"] = self.__git_version
        else:
            self.__labels["version"] = "'{0}@{1} ({2})'".format(
                self.__repo, self.__branch, self.__commit
            )
        self.__labels["cmake_args"] = "'" + " ".join(self.__cmake_args) + "'"

    def runtime(self, _from="0"):
        instructions = [
            comment(
                "OpenGeoSys build from repo {0}, branch {1}".format(
                    self.__repo, self.__branch
                )
            ),
            copy(_from=_from, src=self.__prefix, dest=self.__prefix),
        ]
        if self.__environment_variables:
            instructions.append(environment(variables=self.__environment_variables))
        if self.__labels:
            instructions.append(label(metadata=self.__labels))
        return "\n".join(str(x) for x in instructions)



# Recipe template
# Have a look and change statements with ### SET comment

### SET imports
from hpccm.primitives import comment, shell, raw
from hpccm.building_blocks import packages

print(f"Evaluating {filename}")

# Add cli arguments to args_parser
parse_g = parser.add_argument_group(filename)

### SET arguments, e.g:
parse_g.add_argument("--vscode_user", type=str, default="vscode")

# Parse local args
local_args = parser.parse_known_args()[0]

### SET append to image file name, e.g.:
img_file += f"-vscode"

### SET Append to out_dir, e.g.:
out_dir += f"/vscode"

# Implement recipe
Stage0 += comment(f"Begin {filename}")

username = local_args.vscode_user
id = 1001

Stage0 += packages(ospackages=["sudo", "gdb"])
Stage0 += shell(
    commands=[
        f"groupadd --gid {id} {username}",
        f"useradd --uid {id} --gid {id} -m {username}",
        f"echo {username} ALL=\(root\) NOPASSWD:ALL > /etc/sudoers.d/{username}",
        f"chmod 0440 /etc/sudoers.d/{username}",
    ]
)

Stage0 += raw(docker=f"USER {username}")

Stage0 += comment(f"--- End {filename} ---")
